/*
	COUNTDOWN LIVE - by Mattias Buelens
	File: Language.js
	Desc: Class and functions for translation handling
	Original from Screenshot Sender by Matty, slightly modified by myself
*/
/*
 * ----------------------------------------------------------------------------
 * "THE BEER-WARE LICENSE" (Revision 42):
 * <darktempler@gmail.com> wrote this file. As long as you retain this notice you
 * can do whatever you want with this stuff. If we meet some day, and you think
 * this stuff is worth it, you can buy me a beer in return Matt Labrum (-dt-)
 * ----------------------------------------------------------------------------
 */

//Checks if the language is valid and loaded
function IsLangOK(sLang) {
	if(typeof sLang === "string") {
		var oLang = new Translation(sLang);
		return oLang.IsLoaded();
	} else if(typeof sLang === "object") {
		return sLang.IsLoaded();
	} else return (typeof Lang === "object" && Lang.IsLoaded());
}

//Reloads the language
function ReloadLanguage() {
	SetDateFormatUnits();
	if(IsWindow("WndPref")) var bIsChanged = Temp.WndPref["IsChanged"];
	for(var i in Wnd) {
		if(IsWindow(i)) try {
			Lang.LoadWndLang(Wnd[i], i);
			eval(i+"_ReloadLang(Wnd[i])");
		} catch(e) {
			Trace("! Couldn't reload language for "+i+":\n  "+e.description);
			continue;
		}
	}
	if(IsWindow("WndPref")) Temp.WndPref["IsChanged"] = bIsChanged;
}

//Sets the units for date formatting
function SetDateFormatUnits() {
	try {
		with(Lang.GetBlock("Messages")) {
			Date.monthNames = GetString("Months").split("|");
			Date.dayNames = GetString("Days").split("|");
			Date.dayShortNames = GetString("DaysShort").split("|");
			Date.monthShortNames = GetString("MonthsShort").split("|");
			for(var i=0; i<=11; i++) Date.monthNumbers[Date.monthShortNames[i]] = i;
			Date.daySuffix = GetString("DaysSuffix").split("|");
		}
	} catch(e) {
		Trace("! Error while setting units for date formatting:\n  "+e.description);
	}
}

//Checks the existence of the language file and the version of the chosen language. If the language is OK, the function loads the language.
function CheckLanguage(sLang) {
	var bResult = false;
	var bCheckOnly = (typeof sLang === "string");
	var sLang = bCheckOnly ? sLang : Pref["Lang"];
	//Check if the language file exists
	if(!FileExists(PathAppend(MsgPlus.ScriptFilesPath, "Languages\\" + Pref["Lang"]))) {
		if(bCheckOnly) return false;
		//It doesn't, let's set it to the default
		Pref["Lang"] = Keys["Lang"].Default;
		Registry_SetKeyValue(HKEY_CURRENT_USER, RegPath, "Lang", Pref["Lang"], REG_SZ);
	}

	if(!bCheckOnly) Trace("  Checking language version", 1);
	//Check the language's version
	var oLang = new Translation(Pref["Lang"]);
	if(oLang.IsLoaded()) {
		var bValid = CheckLangVersions(oLang);
		if(bCheckOnly) return bValid;
		if(bValid) {
			Trace("   Language version OK, loading language", 1);
			Registry_DeleteKeyValue(HKEY_CURRENT_USER, RegPath, "NoLangVersionWarning");
			Lang = oLang;
			Trace("   Language loading ended", 1);
			ReloadLanguage();
			bResult = true;
		} else {
			//Check whether the language version popup has already been shown before
			//This popup is user-specific: another user will get a new popup regardless if it was shown before
			Trace("!!! WARNING: Language version doesn't match script version", 1);
			var bShowPopup = (Registry_GetKeyValue(HKEY_CURRENT_USER, RegPath, "NoLangVersionWarning") !== Pref["Lang"]);
			if(bShowPopup) {
				//Show a popup
				var oVersion = oLang.GetVersion();
				var sMsg = ParseMsgVars(
					"The selected language doesn't support this version of Countdown Live.\n\nLanguage: %1\nMinimum version: %2\nMaximum version: %3\nCurrent script version: %4\n\nDo you want to load this language anyway? (not recommended)",
					[Pref["Lang"], oVersion.MinVersion, oVersion.MaxVersion, Version]
				);
				var Result = MsgBox(sMsg, "Language doesn't support current version", MB_YESNO | MB_DEFBUTTON2 | MB_ICONHAND);
			}
			if(!bShowPopup || Result === 6) {
				//Popup was shown - force the language to load
				Trace("  Force language loading", 1);
				Registry_SetKeyValue(HKEY_CURRENT_USER, RegPath, "NoLangVersionWarning", Pref["Lang"], REG_SZ);
				Lang = oLang;
				ReloadLanguage();
				bResult = true;
			}
		}
	}
	if(bCheckOnly) return false;
	Trace("  Language version checking ended", 1);
	return bResult;
}

//Try to find the language file for the current Plus! language, or a given language code
function FindUserLanguage(sLangCode) {
	sLangCode = (typeof sLangCode === "string") ? sLangCode : MsgPlus.UILangCode;

	//Loop through the language files
	var sLangPath = PathAppend(MsgPlus.ScriptFilesPath, "Languages");
	for(var e = new Enumerator(new ActiveXObject("Scripting.FileSystemObject").GetFolder(sLangPath).Files); !e.atEnd(); e.moveNext()) {
		var oFile = e.item();
		if(/\.xml$/i.test(oFile.Name)) {
			var TempLang = new Translation(oFile.Name);
			if(TempLang.GetLangCode() === sLangCode) {
				return oFile.Name;
			}
		}
	}

	return false;
}

/*
	SetDefaultLanguage

	Sets the default language, looking in the languages folder and in the database.
	Runs silently (no error messages) and is supposed to be run once.
*/
function SetDefaultLanguage() {
	var sDefault = "English.xml";
	if(!ComparePlusVersion("4.20.262")) return SetLanguage(sDefault);
	/* STEP 1: Check if the current Plus! language is available in the Languages folder */
	if(MsgPlus.UILangCode === "en") {
		//English file
		return SetLanguage(sDefault);
	}
	var LangFile = FindUserLanguage();
	if(typeof LangFile === "string") {
		//Found the language
		return SetLanguage(LangFile);
	}
	/* STEP 2: Check if there's a language file on the server with the current Plus! language code */
	var sUrl = LangsUrl.replace("{RELEASETYPE}", ReleaseType);
	//Set the XmlHttp request options
	var requestOptions = {
		"xml" : true,
		"ontimeout" : function() {
			oTimer.Cancel();
			xmlHttp.abort();
			return SetLanguage(sDefault);
		}
	};
	//Set the XmlHttp request callback function on success
	var requestCallback = function(xml) {
		//Document is invalid
		if(!xml || xml.documentElement == null) return SetLanguage(sDefault);
		//Look for the language
		var node = xml.documentElement.selectSingleNode("//Language[@Code='" + MsgPlus.UILangCode +"']");
		if(node == null) return SetLanguage(sDefault);
		var sLocation = node.selectSingleNode("//Location").text;
		var sName = node.selectSingleNode("File").text;
		var sDestFolder = PathAppend(GetTempPath(), "Countdown Live");
		if(!IsFolder(sDestFolder)) CreateFolder(sDestFolder);
		var LangDownload = new Download(sLocation + sName, PathAppend(sDestFolder, GetFileName(sName)), OnCompleted);
	}
	//Do the XmlHttp request
	var request = new XmlHttp(sUrl, requestCallback, requestOptions);

	function OnCompleted(sOutFile) {
		if(typeof sOutFile !== "string") {
			DeleteFile(sOutFile);
			return SetLanguage(sDefault);
		}
		var sLangsLoc = PathAppend(MsgPlus.ScriptFilesPath, "Languages");
		var sFileName = GetFileName(sOutFile);
		var sDest = PathAppend(sLangsLoc, sFileName);
		MoveFile(sOutFile, sDest, true);
		return SetLanguage(sFileName);
	}

	function SetLanguage(sLang) {
		Pref["Lang"] = sLang;
		Preferences_Save("Lang");
		Trace("> Language set to: "+sLang, 1);
		return true;
	}
}

//Checks for updates for incompatible languages on the server
function CheckLanguageUpdates(bForced) {
	if(!(bForced || Pref["CheckLangUpdates"])) return false;
	Trace("> Checking for language updates", 1);
	//Collect a list of incompatible languages
	var oLangsToCheck = {};
	var nLangsToCheck = 0;
	var bUILangCompatible = false;
	var sLangPath = PathAppend(MsgPlus.ScriptFilesPath, "Languages");
	for(var e = new Enumerator(new ActiveXObject("Scripting.FileSystemObject").GetFolder(sLangPath).Files); !e.atEnd(); e.moveNext()) {
		var oFile = e.item();
		if(/\.xml$/i.test(oFile.Name)) {
			var TempLang = new Translation(oFile.Name);
			if(TempLang.IsLoaded()) {
				var sLangCode = TempLang.GetLangCode();
				var bIsCompatible = CheckLangVersions(TempLang);
				if(!bIsCompatible) {
					oLangsToCheck[TempLang.GetLangCode()] = 0;
					nLangsToCheck++;
				} else if(sLangCode === MsgPlus.UILangCode) {
					bUILangCompatible = true;
				}
			}
		}
	}
	//Check the amount of incompatible languages
	if(nLangsToCheck > 0) {
		Trace("  Got "+nLangsToCheck+" incompatible language(s)", 1);
	} else {
		Trace("  All languages are compatible!", 1);
		return true;
	}
	//If not added yet, add the Plus language to the list of languages to check
	if(typeof MsgPlus.UILangCode === "string" && !bUILangCompatible) oLangsToCheck[MsgPlus.UILangCode] = 0;
	//Initiate the update check
	Trace("  Getting XML");
	var sUrl = LangsUrl.replace("{RELEASETYPE}", ReleaseType);
	//Set the XmlHttp request options
	var requestOptions = {
		"xml" : true,
		"ontimeout" : function() {
			Trace("> Languages update checking XmlHttp request timed out", 1);
			ShowError("DownLangsXmlHttpTimeout", sUrl, MB_ICONHAND);
		}
	};
	//Set the XmlHttp request callback function on success
	var requestCallback = function(xml) {
		//Request was succesful
		Trace("> Languages update check: Got XML\n  Getting available languages from file");
		//Document is invalid
		if(!xml || xml.documentElement == null) {
			Trace("!!! Error: Couldn't find root node", 1);
			ShowWindow(PlusWnd.GetControlHandle("LblLoading"), false);
			ShowError("DownLangsXmlNoRoot", sUrl, MB_ICONHAND);
			return;
		}
		//Check the versions of the incompatible languages
		var bGotUpdates = false;
		for(var sLangCode in oLangsToCheck) {
			//Get this node's MinVersion and MaxVersion
			var Nodes = xml.selectNodes("/Languages/Language[@Code='"+sLangCode+"']");
			if(Nodes.length < 1) continue;
			var Node = Nodes[0];
			var Version = Node.selectNodes("Version/*");
			if(Version.length < 1) continue;
			var oVersion = {
				MinVersion: Version[0].text,
				MaxVersion: Version[1].text
			};
			//Check if this new language is compatible
			if(CheckLangVersions(oVersion)) {
				//Found an updated version!
				bGotUpdates = true;
				break;
			};
		}
		//Show a toast to notify the user about the new language updates
		if(bGotUpdates) {
			Trace("> Got new language updates!", 1);
			var oToast = (IsLangOK() && Lang.IsValid("Toasts")) ? Lang.GetToast("LangUpdates") : false;
			if(!oToast) oToast = {
				Line1: "[b]Language updates available[/b]",
				Line2: "Click here to download the updated languages."
			};
			MsgPlus.DisplayToastContact("Countdown Live", oToast.Line1, oToast.Line2, 0, "OnEvent_LangUpdatesToastClicked", xml);
		} else Trace("  No new language updates found", 1);
	};
	//Do the XmlHttp request
	var request = new XmlHttp(sUrl, requestCallback, requestOptions);
}

//Custom event when the language updates toast is clicked
function OnEvent_LangUpdatesToastClicked(xml) {
	OpenWndDownloadLangs(xml);
}

/*
	TRANSLATION CLASS
*/
var Translation = function(type) {
	this.xml = CreateXML();
	if(type.indexOf('.xml') === -1) { this.translationFile = type + ".xml";	}
	else { this.translationFile = type; }
	this.LoadTranslation();
}

Translation.prototype = {
	"xml" : false,
	"loaded" : false,
	"translationFile" : "English.xml",
	"LanguageFolder" : AddBackslash(PathAppend(MsgPlus.ScriptFilesPath, "Languages")),

	"IsLoaded" : function() {
		return this.loaded;
	},

	"IsValid" : function(sTag, sId) {
		if(!this.IsLoaded()) return false;
		return (this.xml.selectNodes("/Translation/" + sTag + (typeof sId !== "undefined" ? "[@Id='" + sId + "']" : "")).length > 0);
	},

	"IsRTL" : function() {
		var rtl = this.xml.selectNodes("/Translation/RTL");
		return (rtl.length > 0) ? 1 * rtl[0].text : false;
	},

	"GetName" : function() {
		if(!this.IsLoaded()) return "Unknown";
		var name = this.xml.selectNodes("/Translation/Name");
		return (name.length > 0) ? StripSlashes(name[0].text) : "Unknown";
	},

	"GetAuthor" : function() {
		if(!this.IsLoaded()) return "Unknown";
		var author = this.xml.selectNodes("/Translation/Author");
		return (author.length > 0) ? StripSlashes(author[0].text) : "Unknown";
	},

	"GetWebsite" : function() {
		if(!this.IsLoaded()) return "";
		var website = this.xml.selectNodes("/Translation/Website");
		return (website.length > 0) ? StripSlashes(website[0].text) : "";
	},

	"GetLangCode" : function() {
		if(!this.IsLoaded()) return "";
		var langcode = this.xml.selectNodes("/Translation/LangCode");
		return (langcode.length > 0) ? StripSlashes(langcode[0].text) : "";
	},

	"GetVersion" : function() {
		if(!this.IsLoaded()) return "";
		var returns = new Object();
		var minVersion = this.xml.selectNodes("/Translation/MinVersion");
		returns.MinVersion = (minVersion.length > 0) ? minVersion[0].text : "";
		var maxVersion = this.xml.selectNodes("/Translation/MaxVersion");
		returns.MaxVersion = (maxVersion.length > 0) ? maxVersion[0].text : "";
		return returns;
	},

	"LoadTranslation" : function() {
		try {
			var path = this.LanguageFolder + this.translationFile;
			if(!FileExists(path)) throw new Error("NotFound");
			this.xml.async = false;
	 		this.xml.load(path);
			if(this.xml.parseError.errorCode) throw new Error("Parse");
			this.loaded = true;
			Temp.DateFormat = StripSlashes(this.GetBlock("Messages").GetString("DateFormat"));
		} catch(e) {
			var sPath = this.LanguageFolder + this.translationFile;
			var oError = {};
			switch(e.description) {
				case "NotFound":
					oError.Trace = "Language file not found\nPath: "+sPath;
					oError.Message = "Language file not found!\nPlease re-install the script.\n\nPath: "+sPath;
					oError.Title = "Language file not found\nPath: "+sPath;
					break;
				case "Parse":
					var p = this.xml.parseError;
					var reason = p.reason.replace(/(\n|\r)/g, "");
					oError.Trace = "Error while loading translation:\n" + reason + " (" + p.errorCode + ")\nLine "+p.line+", char "+p.linepos+": "+p.srcText;
					oError.Message = "The language file couldn't be loaded. A parse error occured while loading the XML data on line "+p.line+", character "+p.linepos+":\n\n"+reason+" ("+p.errorCode+")\n\nPath: "+sPath+"\nSource: "+p.srcText;
					oError.Title = "Parse error in language file";
					break;
				default:
					oError.Trace = "Error number: " + e.number + "\n" + e.description;
					oError.Message = "An error occured while loading the language file:\n\n" + e.description;
					oError.Title = "Error while loading language file";
			}
			Trace("!!! Language error: "+oError.Trace.replace(/(\n|\r)/g, "\n    "), 1);
			MsgBox(oError.Message, oError.Title, MB_ICONHAND | MB_TOPMOST);
			this.loaded = false;
			return false;
		}
	},

	"LoadWindow" : function(XmlFile,WindowId,Flags) {
		var wnd = MsgPlus.CreateWnd(XmlFile, WindowId, (typeof Flags === "undefined" ? 1 : Flags));
		this.LoadWndLang(wnd, WindowId);
		if(typeof Flags === "undefined") wnd.Visible = true;
		return wnd;
	},

	"LoadChildWindow" : function(Parent,XmlFile,WindowId,X,Y) {
		var wnd = MsgPlus.CreateChildWnd(Parent, XmlFile, WindowId, X, Y, false);
		this.LoadWndLang(wnd, WindowId);
		wnd.Visible = true;
		return wnd;
	},

	"LoadWndLang" : function(wnd) {
		if(!(this.IsLoaded() && IsWindow(wnd))) return false;
		var wndId = wnd.WindowId;
		var wndXML = this.GetWindow(wndId);
		if(!wndXML) return false;
		var objWnd = wndXML.ToObject();
		for(var x in objWnd) {
			if(typeof objWnd[x] === "string") {
				if(x === "Caption") { if(ComparePlusVersion("4.50.317")) {
					Interop.Call("user32", "SendMessageW",
						wnd.Handle,
						/* WM_SETTEXT */ 0xC,
						/* wParam */ true,
						/* lParam */ StripSlashes(objWnd[x])
					);
				} } else {
					wnd.SetControlText(x, StripSlashes(objWnd[x]));
				}
			} else if(typeof objWnd[x] === "object") {
				for(var i in objWnd[x]) {
					var Result = new ListView(wnd.GetControlHandle(x)).SetColumn(1*i, StripSlashes(objWnd[x][i]));
				}
			}
		}
		return true;
	},

	"GetBlock" : function(strTag, strID) {
		if(!this.IsLoaded()) return false;
		var strTag = (typeof strTag === "string" && strTag.length > 0) ? strTag : "Messages";
		var strID = (typeof strID === "string" && strID.length > 0) ? strID : false;
		var block = this.xml.selectNodes("/Translation/"+strTag+(strID !== false ? "[@Id='"+strID+"']" : ""));
		return (block.length > 0) ? new this.window(block[0]) : false;
	},

	"GetMenu" : function(menuId) {
		return this.GetBlock("Menu", menuId);
	},

	"GetWindow" : function(wndID) {
		return this.GetBlock("Window", wndID);
	},

	"GetLink" : function(lnkID) {
		return this.GetBlock("Links").GetString(lnkID);
	},

	"GetToast" : function(toastID) {
		var toasts = this.GetBlock("Toasts");
		if(!toasts) return false;
		toasts = toasts.ToObject();
		return toasts[toastID];
	},

	"GetEvent" : function(eventID) {
		var events = this.GetBlock("Events");
		if(!events) return false;
		events = events.ToObject();
		return events[eventID];
	},

	"GetSpecialString" : function(name) {
		if(!this.IsLoaded()) return false;
		var elements = this.xml.selectNodes("/Translation/" + name);
		return (elements.length > 0) ? StripSlashes(elements[0].text) : false;
	}
}

Translation.prototype.window = function(windowObj) {
	this.xml = windowObj;
}

Translation.prototype.window.prototype = {
	"GetString" : function(id) {
		var str = this.xml.selectNodes("String[@Id='" + id + "']");
		return (str.length > 0) ? StripSlashes(str[0].text) : false;
	},

	"GetStrings" : function(xml) {
		return new this.StringBundle((typeof xml === "undefined") ? this.xml : xml);
	},

	"GetStringBundle" : function(id) {
		var bundle = this.xml.selectNodes("StringBundle[@Id='" + id + "']");
		return (bundle.length > 0) ? new this.StringBundle(bundle[0]) : false;
	},

	"GetDialog" : function(id, oDefault) {
		var dlg = this.xml.selectNodes("Dialog[@Id='" + id + "']");
		if(dlg.length > 0) dlg = dlg[0]; else return oDefault;
		var childs = dlg.childNodes;
		var returns = {};
		for(var i=0; i<childs.length; i++) {
			var child = childs[i];
			returns[child.tagName] = child.text;
		}
		return returns;
	},

	"ToObject" : function() {
		var elements = this.xml.selectNodes("*");
		var returns = {};
		for(var i=0;i<elements.length;i++) {
			if(elements[i].tagName === "Control") {
				var id = elements[i].getAttribute("Id");
				var childs = elements[i].childNodes;
				var value = false;
				for(var j=0; j<childs.length; j++) {
					if(childs[j].tagName === "Column") {
						if(typeof value !== "object") value = {};
						value[j] = StripSlashes(childs[j].text);
					} else if(childs[j].tagName === "Link") {
						value = Lang.GetLink(childs[j].text);
					}
				}
				if(value === false) returns[id] = StripSlashes(elements[i].text);
				else returns[id] = value;
			} else if(elements[i].tagName === "ToolTip") {
				var id = elements[i].getAttribute("Id");
				var obj = {};
				var childs = elements[i].childNodes;
				for(var j=0; j<childs.length; j++) {
					if(childs[j].tagName === "Link") {
						obj["Body"] = Lang.GetLink(childs[j].text);
					} else {
						obj[childs[j].tagName] = StripSlashes(childs[j].text);
					}
				}
				if(typeof obj.Title === "undefined") obj.Title = "";
				if(typeof obj.Icon === "undefined") obj.Icon = "NONE";
				else if(/^TTI_/.test(obj.Icon)) obj.Icon = obj.Icon.substr("TTI_".length);
				returns[id] = obj;
			} else if(elements[i].tagName === "Error") {
				var id = elements[i].getAttribute("Id");
				var obj = {};
				var childs = elements[i].childNodes;
				for(var j=0; j<childs.length; j++) {
					obj[childs[j].tagName] = StripSlashes(childs[j].text);
				}
				if(typeof obj.Title === "undefined") obj.Title = "Error";
				returns[id] = obj;
			} else if(elements[i].tagName === "SubMenu") {
				var label = elements[i].selectNodes("Label");
				label = (label.length > 0) ? StripSlashes(label.item(0).text) : "<Unknown>";
				var obj = this.GetStrings(elements[i]).ToObject();
				returns[label] = obj;
			} else if(elements[i].nodeType === 1) {
				if(elements[i].tagName === "String") {
					returns[elements[i].getAttribute('Id')] = StripSlashes(elements[i].text);
				} else if(elements[i].tagName === "StringBundle") {
					var bundle = new this.StringBundle(elements[i]);
					returns[elements[i].getAttribute('Id')] = bundle.GetDefault();
				} else if(elements[i].tagName === "Separator") {
					returns[elements[i].getAttribute('Id')] = MF_SEPARATOR;
				} else if(elements[i].hasChildNodes()) {
					var sId = elements[i].getAttribute('Id') || elements[i].tagName;
					var childs = elements[i].childNodes;
					if(childs.length === 1 && childs[0].nodeType === 3) {
						returns[sId] = childs[0].text;
					} else {
						returns[sId] = new Translation.prototype.window(elements[i]).ToObject();
					}
				} else {
					returns[elements[i].tagName] = StripSlashes(elements[i].text);
				}
			}
		}
		return returns;
	},

	"StringBundle" : function(bundleObj) {
		this.xml = bundleObj;
	}
}

Translation.prototype.window.prototype.StringBundle.prototype = {
	"ToObject" : function() {
		var nodes = this.xml.selectNodes("String");
		var returns = {};
		for(var i=0;i<nodes.length;i++) {
			if(nodes[i].nodeType === 1) {
				returns[nodes[i].getAttribute("Id")] = StripSlashes(nodes[i].text);
			}
		}
		return returns;
	},

	"GetDefault" : function() {
		var str = this.xml.selectNodes("Default");
		return (str.length > 0) ? StripSlashes(str[0].text) : false;
	},

	"GetString" : function(id) {
		var str = this.xml.selectNodes("String[@Id='" + id + "']");
		return (str.length > 0) ? StripSlashes(str[0].text) : false;
	}
}